
//////////////////////////////////////////////////
//  Oracle Data Access Components
//  Copyright  1998-2024 Devart. All right reserved.
//////////////////////////////////////////////////

{$IFNDEF CLR}

{$I Odac.inc}

unit OraDesignUtils;
{$ENDIF}

interface

uses
{$IFDEF DBTOOLS}
  DBToolsClient{$IFDEF VER11P}, DBToolsClientImp{$ENDIF}, DBForgeStudioClientImp,
  DBToolsIntf, DB, DesignIntf,
{$ENDIF}
  Classes, SysUtils, CRTypes, CRDataTypeMap, DBAccess,
  CRDesignUtils, DADesignUtils;

type

{$IFDEF DBTOOLS}
  TOraIntegrationTool = ({$IFDEF VER11P}{$IFNDEF VER17P}oitOraDeveloperTools, {$ENDIF}{$ENDIF}oitDBForgeStudio, oitNone);
{$ENDIF}

  TOraDesignUtils = class(TDADesignUtils)
    class function GetProjectName: string; override;

  { Component }
    class function GetDesignCreate(Obj: TComponent): boolean; override;
    class procedure SetDesignCreate(Obj: TComponent; Value: boolean); override;
  {$IFDEF ODAC}
    class function GetConnectionList: TObject; override;
  {$ENDIF}

  { Connection support }
    class function HasConnection(Obj: TComponent): boolean; override;
    class function GetConnection(Obj: TComponent): TCustomDAConnection; override;
    class procedure SetConnection(Obj: TComponent; Value: TCustomDAConnection); override;
    class function UsedConnection(Obj: TComponent): TCustomDAConnection; override;

  { TDATable support }
    class function GetTableName(Obj: TCustomDADAtaSet): string; override;
    class procedure SetTableName(Obj: TCustomDADAtaSet; const Value: string); override;
    class function GetOrderFields(Obj: TCustomDADAtaSet): string; override;
    class procedure SetOrderFields(Obj: TCustomDADAtaSet; const Value: string); override;
    class procedure PrepareSQL(Obj: TCustomDADAtaSet); override;

  { TDAStoredProc support }
    class function GetStoredProcName(Obj: TCustomDADataSet): string; override;
    class procedure SetStoredProcName(Obj: TCustomDADataSet; const Value: string); override;

  { Data Type Mapping }
    class function GetConverterManagerClass: TConverterManagerClass; override;

  {$IFDEF USE_SYNEDIT}
    class function SQLDialect: integer ; override; // SynHighlighterSQL TSQLDialect = (sqlStandard, sqlInterbase6, sqlMSSQL7, sqlMySQL, sqlOracle, sqlSybase, sqlIngres, sqlMSSQL2K);
  {$ENDIF}
  {$IFDEF DBTOOLS}
    class function DBToolsService: TObject; override;
    class function NeedToCheckDbTools: TNeedToCheckDbTools; override;
    class function GetDBToolsServiceVersion: int64; override;
    class function GetDBToolsMenuCaption: string; override;
    class function GetFullName(Obj: TComponent): string; override;
    class function GetObjectType(Obj: TComponent): string; override;
    class function IsStoredProc(Obj: TComponent): boolean; override;

    class procedure GetDBToolsConnectionList(Connection: TCustomDAConnection); override;
    class procedure SetDBToolsDownloadParams(VerbCheck: boolean; Incompatible: boolean); override;

    class procedure LoadIntegrationTool;
    class procedure SaveIntegrationTool(IntegrationTool: TOraIntegrationTool);
    class function GetCurrentIntegrationTool: TOraIntegrationTool;
    class procedure UpdateIntegrationTool;
    class function AreDBToolsUsed: boolean; override;
    class procedure CheckComponent(Component: TComponent); override;
  {$ENDIF}
  end;

  TOraDesignUtilsClass = class of TOraDesignUtils;

{$IFDEF DBTOOLS}
{$I OdacVer.inc}

const
  sOraIntegrationTool = 'OraIntegrationTool';
{$IFDEF VER11P}{$IFNDEF VER17P}
  sOraDeveloperTools = 'OraDeveloper Tools';
{$ENDIF}{$ENDIF}
  sDBForgeStudioTool = 'dbForge Studio for Oracle';
  sNoTool = 'No integration';
  sDBToolsNames: array[TOraIntegrationTool] of string = ({$IFDEF VER11P}{$IFNDEF VER17P}sOraDeveloperTools, {$ENDIF}{$ENDIF}sDBForgeStudioTool, '');

resourcestring
  sDevToolsDownloadCaption = 'ODAC Information';
  sOdacVersion = 'ODAC ' + ODACVersion;
  sDevAskIncompatible = 'DevAskIncompatible';
  sDevAskNoAddin = 'DevAskNoAddin';
  sProductAskIncompatible = 'OdacAskIncompatible';
{$ENDIF}

implementation

uses
{$IFDEF DBTOOLS}
  Download, Windows, Registry,
{$ENDIF}
{$IFDEF ODAC}
  OraDesign,
{$ENDIF}
{$IFNDEF FPC}MemDS{$ELSE}MemDataSet{$ENDIF},
  OraCall, OraClasses, Ora, OraSmart, OraDataTypeMap,
{$IFNDEF STD}
  OraPackage, OraErrHand, OraAQ, OraTransaction,
{$ENDIF}
  CRFunctions;

{$IFDEF DBTOOLS}

{$I IdeConsts.inc}

var
  OracleService: TObject;
  OracleServiceFault: boolean;
  OracleServiceNeedToCheck: TNeedToCheckDbTools;
  OracleServiceVersion: int64;
  OracleIntegrationTool: TOraIntegrationTool;
  OracleServiceGUID: TGUID;
  OracleProviderKey: string;

{$ENDIF}

{ TOraDesignUtils }

class function TOraDesignUtils.GetProjectName: string;
begin
  Result := 'ODAC';
end;

class function TOraDesignUtils.GetDesignCreate(Obj: TComponent): boolean;
begin
  Assert(Obj <> nil);
{$IFNDEF STD}
  if Obj is TOraErrorHandler then
    Result := TOraErrorHandlerUtils.GetDesignCreate(TOraErrorHandler(Obj))
  else
  if Obj is TCustomOraPackage then
    Result := TOraPackageUtils.GetDesignCreate(TCustomOraPackage(Obj))
  else
  if Obj is TOraQueue then
    Result := TOraAQUtils.GetDesignCreate(TOraQueue(Obj))
  else
  if Obj is TOraQueueTable then
    Result := TOraAQUtils.GetDesignCreate(TOraQueueTable(Obj))
  else
  if Obj is TOraQueueAdmin then
    Result := TOraAQUtils.GetDesignCreate(TOraQueueAdmin(Obj))
  else
  if Obj is TOraTrace then
    Result := TOraUtils.GetDesignCreate(TOraTrace(Obj))
  else
{$ENDIF}
    Result := inherited GetDesignCreate(Obj);
end;

class procedure TOraDesignUtils.SetDesignCreate(Obj: TComponent; Value: boolean);
begin
  Assert(Obj <> nil);
{$IFNDEF STD}
  if Obj is TOraErrorHandler then
    TOraErrorHandlerUtils.SetDesignCreate(TOraErrorHandler(Obj), Value)
  else
  if Obj is TCustomOraPackage then
    TOraPackageUtils.SetDesignCreate(TCustomOraPackage(Obj), Value)
  else
  if Obj is TOraQueue then
    TOraAQUtils.SetDesignCreate(TOraQueue(Obj), Value)
  else
  if Obj is TOraQueueTable then
    TOraAQUtils.SetDesignCreate(TOraQueueTable(Obj), Value)
  else
  if Obj is TOraQueueAdmin then
    TOraAQUtils.SetDesignCreate(TOraQueueAdmin(Obj), Value)
  else
  if Obj is TOraTrace then
    TOraUtils.SetDesignCreate(TOraTrace(Obj), Value)
  else
{$ENDIF}
    inherited;
end;

{$IFDEF ODAC}
class function TOraDesignUtils.GetConnectionList: TObject;
begin
  Result := TOraConnectionList.Create;
end;
{$ENDIF}

class function TOraDesignUtils.HasConnection(Obj: TComponent): boolean;
begin
  Result := inherited HasConnection(Obj)
  {$IFNDEF STD}
    or (Obj is TOraErrorHandler)
    or (Obj is TCustomOraPackage)
    or (Obj is TOraQueue) or (Obj is TOraQueueTable) or (Obj is TOraQueueAdmin)
    or (Obj is TOraTrace);
  {$ENDIF}  
end;

class function TOraDesignUtils.GetConnection(Obj: TComponent): TCustomDAConnection;
begin
  Assert(Obj <> nil);
{$IFNDEF STD}
  if Obj is TOraErrorHandler then
    Result := TOraErrorHandler(Obj).Session
  else
  if Obj is TCustomOraPackage then
    Result := TCustomOraPackage(Obj).Session
  else
  if Obj is TOraQueue then
    Result := TOraQueue(Obj).Session
  else
  if Obj is TOraQueueTable then
    Result := TOraQueueTable(Obj).Session
  else
  if Obj is TOraQueueAdmin then
    Result := TOraQueueAdmin(Obj).Session
  else
  if Obj is TOraTrace then
    Result := TOraTrace(Obj).Session
  else
{$ENDIF}  
    Result := inherited GetConnection(Obj);
end;

class procedure TOraDesignUtils.SetConnection(Obj: TComponent; Value: TCustomDAConnection);
begin
  Assert(Obj <> nil);
  if Value <> nil then
    Assert(Value is TOraSession);
{$IFNDEF STD}
  if Obj is TOraErrorHandler then
    TOraErrorHandler(Obj).Session := TOraSession(Value)
  else
  if Obj is TCustomOraPackage then
    TCustomOraPackage(Obj).Session := TOraSession(Value)
  else
  if Obj is TOraQueue then
    TOraQueue(Obj).Session := TOraSession(Value)
  else
  if Obj is TOraQueueTable then
    TOraQueueTable(Obj).Session := TOraSession(Value)
  else
  if Obj is TOraQueueAdmin then
    TOraQueueAdmin(Obj).Session := TOraSession(Value)
  else
  if Obj is TOraTrace then
    TOraTrace(Obj).Session := TOraSession(Value)
  else
{$ENDIF}
    inherited;
end;

class function TOraDesignUtils.UsedConnection(Obj: TComponent): TCustomDAConnection;
begin
{$IFNDEF STD}
  if Obj is TCustomOraPackage then
    Result := TOraPackageUtils.UsedConnection(TCustomOraPackage(Obj))
  else
{$ENDIF}
    Result := inherited UsedConnection(Obj);
end;

class function TOraDesignUtils.GetTableName(Obj: TCustomDADAtaSet): string;
begin
  Assert(Obj is TOraTable, Obj.ClassName);
  Result := TOraTable(Obj).TableName;
end;

class procedure TOraDesignUtils.SetTableName(Obj: TCustomDADAtaSet;
  const Value: string);
begin
  Assert(Obj is TOraTable, Obj.ClassName);
  TOraTable(Obj).TableName := Value;
end;

class procedure TOraDesignUtils.PrepareSQL(Obj: TCustomDADAtaSet);
begin
  Assert(Obj is TOraDataSet, Obj.ClassName);
  TOraDataSet(Obj).Prepare;
end;

class function TOraDesignUtils.GetStoredProcName(Obj: TCustomDADataSet): string;
begin
  Assert(Obj is TOraStoredProc, Obj.ClassName);
  Result := TOraUtils.GetFinalSPName(TOraStoredProc(Obj).StoredProcName, TOraStoredProc(Obj).Overload);
end;

class procedure TOraDesignUtils.SetStoredProcName(Obj: TCustomDADataSet; const Value: string);
begin
  Assert(Obj is TOraStoredProc, Obj.ClassName);
  TOraStoredProc(Obj).StoredProcName := Value;
end;

class function TOraDesignUtils.GetConverterManagerClass: TConverterManagerClass;
begin
  Result := TConverterManagerClass(TOraMapRules.GetConverterManager.ClassType);
end;

{$IFDEF USE_SYNEDIT}
class function TOraDesignUtils.SQLDialect: integer;
begin
  Result := 1;
end;
{$ENDIF}

class function TOraDesignUtils.GetOrderFields(
  Obj: TCustomDADAtaSet): string;
begin
  Assert(Obj is TOraTable, Obj.ClassName);
  Result := TOraTable(Obj).OrderFields;
end;

class procedure TOraDesignUtils.SetOrderFields(Obj: TCustomDADAtaSet;
  const Value: string);
begin
  Assert(Obj is TOraTable, Obj.ClassName);
  TOraTable(Obj).OrderFields := Value;
end;

{$IFDEF DBTOOLS}

class function TOraDesignUtils.DBToolsService: TObject;
const
  dbtArray = 1;
  dbtBFile = 2;
  dbtBlob = 3;
  dbtBoolean = 4;
  dbtChar = 5;
  dbtClob = 6;
  dbtCursor = 7;
  dbtDate = 8;
  dbtDouble = 9;
  dbtFloat = 10;
  dbtInteger = 11;
  dbtIntervalDS = 12;
  dbtIntervalYM = 13;
  dbtLong = 14;
  dbtLongRaw = 15;
  dbtNChar = $10;
  dbtNClob = $11;
  dbtNumber = $13;
  dbtNVarChar = $12;
  dbtObject = 20;
  dbtRaw = $16;
  dbtRef = $15;
  dbtRowId = $17;
  dbtTable = $18;
  dbtTimeStamp = $19;
  dbtTimeStampLTZ = $1a;
  dbtTimeStampTZ = $1b;
  dbtVarChar = $1c;
  dbtXml = $1d;
var
  DBToolsServiceClass: TCustomDBToolsServiceClass;
begin
  if (OracleIntegrationTool <> oitNone) and (OracleService = nil) and not OracleServiceFault then begin
    OracleServiceFault := true;    
    {$IFDEF VER11P}{$IFNDEF VER17P}
    if OracleIntegrationTool = oitOraDeveloperTools then
      DBToolsServiceClass := TDBToolsService
    else
    {$ENDIF}{$ENDIF}
    if OracleIntegrationTool = oitDBForgeStudio then
      DBToolsServiceClass := TDBForgeStudioService
    else
      DBToolsServiceClass := nil;
    if DBToolsServiceClass <> nil then begin
      OracleService := DBTools.CreateDBToolsService(DBToolsServiceClass, TOraDesignUtils,
        OracleServiceGUID,
        'Server=;User Id=;Password=;Sid=;Service Name=;Port=1521;Home=;'
        + 'Direct=False;Unicode=False;Connect Mode=Default',
        OracleProviderKey, OracleServiceVersion, OracleServiceNeedToCheck);
      OracleServiceFault := OracleService = nil;
      if not OracleServiceFault then
        with TCustomDBToolsService(OracleService) do begin
          AddParamTypeMap(ftString, dbtVarChar);
          AddParamTypeMap(ftString, dbtRowId);
          AddParamTypeMap(ftInteger, dbtInteger);
          AddParamTypeMap(ftFloat, dbtFloat);
          AddParamTypeMap(ftFloat, dbtDouble);
          AddParamTypeMap(TFieldType(ftNumber), dbtNumber);
          AddParamTypeMap(ftDateTime, dbtDate);
          AddParamTypeMap(ftFixedChar, dbtChar);
          AddParamTypeMap(ftWideString, dbtNVarChar);
          AddParamTypeMap(TFieldType(ftFixedWideChar), dbtNChar);
          AddParamTypeMap(ftMemo, dbtLong);
          AddParamTypeMap(ftBlob, dbtBlob);
          AddParamTypeMap(ftBlob, dbtLongRaw);
          AddParamTypeMap(ftOraClob, dbtClob);
          AddParamTypeMap(ftOraClob, dbtNClob);
          AddParamTypeMap(ftOraBlob, dbtBlob);
          AddParamTypeMap(ftCursor, dbtCursor);
          AddParamTypeMap(ftObject, dbtObject);
          AddParamTypeMap(TFieldType(ftXML), dbtXML);
          AddParamTypeMap(ftReference, dbtRef);
          AddParamTypeMap(ftArray, dbtArray);
          AddParamTypeMap(ftTable, dbtTable);
          AddParamTypeMap(TFieldType(ftBFile), dbtBFile);
          AddParamTypeMap(ftBoolean, dbtBoolean);
          AddParamTypeMap(ftTimeStamp, dbtTimeStamp);
          AddParamTypeMap(TFieldType(ftOraTimestamp), dbtTimeStamp);
          AddParamTypeMap(TFieldType(ftOraTimestampTZ), dbtTimeStampTZ);
          AddParamTypeMap(TFieldType(ftOraTimestampLTZ), dbtTimeStampLTZ);
          AddParamTypeMap(TFieldType(ftIntervalYM), dbtIntervalYM);
          AddParamTypeMap(TFieldType(ftIntervalDS), dbtIntervalDS);
          AddParamTypeMap(ftVarBytes, dbtRaw);        
        end;
    end;
  end;
  Result := OracleService;
end;

class function TOraDesignUtils.NeedToCheckDbTools: TNeedToCheckDbTools;
begin
  DBToolsAvailable;
  Result := OracleServiceNeedToCheck;
end;

class function TOraDesignUtils.GetDBToolsServiceVersion: int64;
begin
  DBToolsAvailable;
  Result := OracleServiceVersion;
end;

class function TOraDesignUtils.GetDBToolsMenuCaption: string;
begin
  Result := sDBToolsNames[TOraDesignUtils.GetCurrentIntegrationTool];
end;

class procedure TOraDesignUtils.GetDBToolsConnectionList(Connection: TCustomDAConnection);
var
{$IFDEF NET}
  ServerInfo: TDirectServerInfo;
  Port: integer;
{$ENDIF}
  HostStr, PortStr, SIDStr, ServiceNameStr: string;
  Service: TCustomDBToolsService;
begin
  Assert(OracleService <> nil);
  Service := TCustomDBToolsService(OracleService);
  if Connection <> nil then
    with TOraSession(Connection) do begin
    {$IFDEF NET}
      if Options.Direct then begin
        ServerInfo := TDirectServerInfo.Create;
        try
          ServerInfo.FullInfo := True;
          ServerInfo.SetServerInfo(Server);
          HostStr := ServerInfo.Host;
          if not TryStrToInt(ServerInfo.Port, Port) then
            raise Exception.Create('Invalid port: ' + ServerInfo.Port);
          PortStr := ServerInfo.Port;
          SIDStr := ServerInfo.SID;
          ServiceNameStr := ServerInfo.ServiceName;
        finally
          ServerInfo.Free;
        end;
      end
      else
    {$ENDIF}
      begin
        HostStr := Server;
        PortStr := '';
        SIDStr := '';
        ServiceNameStr := '';
      end;
      Service.PutConnectionParam(HostStr); // Server
      Service.PutConnectionParam(Username); // User Id
      Service.PutConnectionParam(Password, cfNone); // Password
      if Options.Direct then begin
        if SIDStr <> '' then begin
          Service.PutConnectionParam(SIDStr); // Sid
          Service.SkipConnectionParams(1); // skip Service Name
        end
        else begin
          Service.SkipConnectionParams(1); // skip Sid
          Service.PutConnectionParam(ServiceNameStr); // Service Name
        end;
        Service.PutConnectionParam(PortStr);//Port
        Service.SkipConnectionParams(1); // skip Home
      end
      else begin
        Service.SkipConnectionParams(3); // skip Sid, Service Name, Port
        Service.PutConnectionParam(HomeName); // Home
      end;
      Service.PutConnectionParam(BoolToStr(Options.Direct, True)); // Direct
      Service.PutConnectionParam(BoolToStr(Options.UseUnicode, True)); // Unicode
      case ConnectMode of // Connect Mode
        cmNormal:
          Service.PutConnectionParam('Default');
        cmSysOper:
          Service.PutConnectionParam('SysOper');
        cmSysDBA:
          Service.PutConnectionParam('SysDba');
      end;
    end;
end;

class function TOraDesignUtils.GetFullName(Obj: TComponent): string;
begin
  if Obj is TOraTable then
    Result := GetTableName(TCustomDADataSet(Obj))
  else
  if Obj is TOraStoredProc then begin
    Result := Trim(TOraStoredProc(Obj).StoredProcName);
    if (Result <> '') and (TOraStoredProc(Obj).Overload > 0) then
      Result := Result + ':' + IntToStr(TOraStoredProc(Obj).Overload);
  end
{$IFNDEF STD}
  else
  if Obj is TCustomOraPackage then
    Result := TCustomOraPackage(obj).PackageName
{$ENDIF}    
  else
    Result := inherited GetFullName(Obj);
end;

class function TOraDesignUtils.GetObjectType(Obj: TComponent): string;
begin
  if Obj is TOraTable then
    Result := 'Table,View'
  else
  if Obj is TOraStoredProc then
    Result := 'Procedure,Function'
{$IFNDEF STD}
  else
  if Obj is TCustomOraPackage then
    Result := 'Package'
{$ENDIF}
  else
    Result := inherited GetObjectType(Obj);
end;

class function TOraDesignUtils.IsStoredProc(Obj: TComponent): boolean;
begin
  Result := Obj is TOraStoredProc;
end;

class procedure TOraDesignUtils.SetDBToolsDownloadParams(VerbCheck: boolean; Incompatible: boolean);
var
  AskStr, AtomName, UrlStr, UrlExeStr, ToolsNameStr: string;
{$IFDEF VER11P}{$IFNDEF VER17P}
  OutdatedProduct: boolean;
{$ENDIF}{$ENDIF}
begin
{$IFDEF VER11P}{$IFNDEF VER17P}
  if OracleIntegrationTool = oitOraDeveloperTools then begin
    OutdatedProduct := Incompatible and ((GetDBToolsServiceVersion = 0) or
      (GetDBToolsServiceVersion > 2 * $1000000000000{2.0.0.0}));
    if VerbCheck then begin
      AskStr := '_Verb';
      AtomName := '';
    end
    else begin
      AskStr := '';
      if OutdatedProduct then
        AtomName := 'CheckForOdac atom'
      else
        AtomName := 'CheckForOraDevTools atom'
    end;
    if OutdatedProduct then
      SetToolsCheckingParams(sDevToolsDownloadCaption, GetProjectName, sOraDeveloperTools,
        sProductAskIncompatible + AskStr, sProductAskIncompatible + AskStr,
        'Software\Devart\Odac', 'Odac', 'OraDeveloper',
        'www.devart.com/odac', 'odac' + IntToStr(IDEInfos[IDEVer].Version) + '.exe',
        AtomName)
    else begin
      UrlStr := 'www.devart.com/dbforge/oracle/oradevtools/bds';
      UrlExeStr := 'oradevbds' + IntToStr(IDEInfos[IDEVer].BdsVersion) + '.exe';
      ToolsNameStr := sOraDeveloperTools;
      SetToolsCheckingParams(sDevToolsDownloadCaption, ToolsNameStr, sOdacVersion,
        sDevAskIncompatible + AskStr, sDevAskNoAddin + AskStr,
        'Software\Devart\Odac', 'Odac', 'OraDeveloper',
        UrlStr, UrlExeStr,
        AtomName);
    end;
  end
  else
{$ENDIF}{$ENDIF}
  if OracleIntegrationTool = oitDBForgeStudio then begin
    if VerbCheck then begin
      AskStr := '_Verb';
      AtomName := '';
    end
    else begin
      AskStr := '';
      AtomName := 'CheckForDBForgeOracle atom';
    end;
    UrlStr := 'www.devart.com/dbforge/oracle/studio';
    UrlExeStr := 'dbforgeoracle.exe';
    ToolsNameStr := sDBForgeStudioTool; 
    SetToolsCheckingParams(sDevToolsDownloadCaption, ToolsNameStr, sOdacVersion,
      sDevAskIncompatible + AskStr, sDevAskNoAddin + AskStr,
      'Software\Devart\Odac', 'Odac', 'dbForge Studio',
      UrlStr, UrlExeStr,
      AtomName);
  end;
end;

class procedure TOraDesignUtils.LoadIntegrationTool;
var
  reg: TRegistry;
  ts: string;
begin
  OracleIntegrationTool := oitDBForgeStudio;
{$IFDEF VER11P}{$IFNDEF VER17P}
  OracleIntegrationTool := oitOraDeveloperTools;
{$ENDIF}{$ENDIF}
  reg := TRegistry.Create(KEY_READ OR KEY_WRITE);
  try
    reg.RootKey := HKEY_CURRENT_USER;
    if reg.OpenKey('\SOFTWARE\Devart\' + TOraDesignUtils.GetProjectName, False) then begin
      ts := reg.ReadString(sOraIntegrationTool);
{$IFDEF VER11P}{$IFNDEF VER17P}
      if AnsiSameText(ts, sOraDeveloperTools) then
        OracleIntegrationTool := oitOraDeveloperTools
      else
{$ENDIF}{$ENDIF}
      if AnsiSameText(ts, sDBForgeStudioTool) then
        OracleIntegrationTool := oitDBForgeStudio
      else
      if AnsiSameText(ts, sNoTool) then
        OracleIntegrationTool := oitNone;
    end;
  finally
    reg.Free;
  end;
end;
                               
class procedure TOraDesignUtils.SaveIntegrationTool(IntegrationTool: TOraIntegrationTool);
var
  reg: TRegistry;
  ts: string;
begin
  if IntegrationTool = OracleIntegrationTool then exit;
  reg := TRegistry.Create(KEY_READ OR KEY_WRITE);
  try
{$IFDEF VER11P}{$IFNDEF VER17P}
    if IntegrationTool = oitOraDeveloperTools then
      ts := sOraDeveloperTools
    else
{$ENDIF}{$ENDIF}
    if IntegrationTool = oitDBForgeStudio then
      ts := sDBForgeStudioTool
    else
      ts := sNoTool;
    reg.RootKey := HKEY_CURRENT_USER;
    reg.OpenKey('\SOFTWARE\Devart\' + TOraDesignUtils.GetProjectName, True);
    reg.WriteString(sOraIntegrationTool, ts);
    OracleIntegrationTool := IntegrationTool;

    FreeAndNil(OracleService);
    OracleServiceFault := False;
    OracleServiceNeedToCheck := ncNone;
    OracleServiceVersion := 0;
    UnregisterDesignNotification(DBTools.DesignNotification);
    DBTools.DesignNotification := nil;
    TOraDesignUtils.UpdateIntegrationTool;
  finally
    reg.Free;
  end;
end;

class function TOraDesignUtils.GetCurrentIntegrationTool: TOraIntegrationTool;
begin
  Result := OracleIntegrationTool;
end;

class procedure TOraDesignUtils.UpdateIntegrationTool;
const
{$IFDEF VER11P}
  OracleServiceCLSID_Prefix: string = '{6244D51C-2431-4929-9BDC-F334C9ABD';
  OraProviderKey = '{CF3D7701-0F82-41f6-A83E-A4E9A71BD3AB}';
{$ENDIF}
  OracleServiceCLSID_DBForgeStudio: TGUID = '{6DD058F0-77F4-41F4-B32A-8C14E3E80EDA}';
  OraProviderKey_DBForgeStudio = 'dbForge Studio for Oracle';
begin
{$IFDEF VER11P}{$IFNDEF VER17P}
  if OracleIntegrationTool = oitOraDeveloperTools then begin
    if IDEInfos[IDEVer].Version >= 14 then
      OracleServiceGUID := StringToGUID(OracleServiceCLSID_Prefix + IntToStr(IDEInfos[IDEVer].Version - 1) + '0}')
    else
      OracleServiceGUID := StringToGUID(OracleServiceCLSID_Prefix + IntToStr(IDEInfos[IDEVer].Version) + '0}');
    OracleProviderKey := OraProviderKey;
  end
  else
{$ENDIF}{$ENDIF}
  if OracleIntegrationTool = oitDBForgeStudio then begin
    OracleServiceGUID := OracleServiceCLSID_DBForgeStudio;
    OracleProviderKey := OraProviderKey_DBForgeStudio;
  end;
  if OracleIntegrationTool <> oitNone then
    DBTools.CheckDevTools(OracleServiceGUID, OracleProviderKey, TOraDesignUtils, OracleServiceVersion);
end;

class function TOraDesignUtils.AreDBToolsUsed: boolean;
begin
  Result := OracleIntegrationTool <> oitNone;
end;

class procedure TOraDesignUtils.CheckComponent(Component: TComponent);
begin
  if (Component <> nil) then begin
  {$IFNDEF STD}
    if (Component is TOraPackage) and (TOraPackage(Component).PackageName = '') then
      DatabaseError('Package name is not defined')
    else
  {$ENDIF}
    if (Component is TOraTable) and (TOraTable(Component).TableName = '') then
      DatabaseError('Table name is not defined')
    else
    if (Component is TOraStoredProc) and (TOraStoredProc(Component).StoredProcName = '') then
      DatabaseError('Stored procedure name is not defined')
    else
    if (Component is TOraUpdateSQL) and (TOraUpdateSQL(Component).DataSet = nil) then
      DatabaseError('There is no linked dataset');
  end;
end;

{$ENDIF}

initialization
{$IFDEF DBTOOLS}
  TOraDesignUtils.LoadIntegrationTool;
  TOraDesignUtils.UpdateIntegrationTool;
{$ENDIF}

finalization
{$IFDEF DBTOOLS}
  FreeAndNil(OracleService);
{$ENDIF}

end.
