
//////////////////////////////////////////////////
//  Oracle Data Access Components
//  Copyright  1998-2024 Devart. All right reserved.
//  OraErrorHandler Editor
//////////////////////////////////////////////////

{$IFNDEF CLR}
{$I Odac.inc}

unit OraErrorHandlerEditor;
{$ENDIF}
interface
uses
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, DBCtrls, StdCtrls, Buttons, ToolWin, Grids, DBGrids,
{$IFDEF FPC}
  LResources,
{$ENDIF}
  CREditor, Db, OraErrHand, Ora, OraSmart;

type
  TOraErrorHandlerEditorForm = class(TCREditorForm)
    ToolBar: TToolBar;
    btCreateTable: TSpeedButton;
    DBNavigator: TDBNavigator;
    ToolButton1: TToolButton;
    btGenarateScript: TSpeedButton;
    btActive: TSpeedButton;
    PageControl: TPageControl;
    shGrid: TTabSheet;
    shMemo: TTabSheet;
    DBGrid: TDBGrid;
    ToolBar1: TToolBar;
    btCloseMemo: TSpeedButton;
    DataSource: TDataSource;
    Memo: TMemo;
    procedure btGenarateScriptClick(Sender: TObject);
    procedure btCreateTableClick(Sender: TObject);
    procedure btActiveClick(Sender: TObject);
    procedure btCloseMemoClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);

  private
    FErrorHandler: TOraErrorHandler;
    FQuery: TSmartQuery;

    procedure CheckTableName;

    procedure SetErrorHandler(Value: TOraErrorHandler);

  protected

    procedure DoInit; override;
    procedure DoSave; override;
    procedure DoFinish; override;

    function GetComponent: TComponent; override;
    procedure SetComponent(Value: TComponent); override;

  public

    property ErrorHandler: TOraErrorHandler write SetErrorHandler;
  end;

implementation

{$IFNDEF FPC}
{$R OraErrorHandlerEditor.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

procedure TOraErrorHandlerEditorForm.DoInit;
begin
  inherited;

{$IFDEF VER27P}
  DBNavigator.Flat := False;
{$ENDIF}

  Caption := FErrorHandler.Owner.Name + '.' + FErrorHandler.Name;
  FQuery := TSmartQuery.Create(nil);
  FQuery.Session := FErrorHandler.Session;
  FQuery.RefreshOptions := [];
  FQuery.LockMode := lmNone;

  DataSource.DataSet := FQuery;

  FQuery.SQL.Text := 'SELECT T.*,RowId FROM ' + FErrorHandler.TableName + ' T';

  if not FErrorHandler.Active then
    btActive.Caption := 'Open'
  else
    btActive.Caption := 'Close';

  FQuery.Active := FErrorHandler.Active;
end;

procedure TOraErrorHandlerEditorForm.DoSave;
begin
end;

procedure TOraErrorHandlerEditorForm.DoFinish;
begin
  if FErrorHandler.Session <> nil then
    FErrorHandler.Session.Commit;
  FQuery.Free;
end;

function TOraErrorHandlerEditorForm.GetComponent: TComponent;
begin
  Result := FErrorHandler;
end;

procedure TOraErrorHandlerEditorForm.SetComponent(Value: TComponent);
begin
  FErrorHandler := Value as TOraErrorHandler;
end;

procedure TOraErrorHandlerEditorForm.SetErrorHandler(Value: TOraErrorHandler);
begin
  FErrorHandler := Value;
end;

procedure TOraErrorHandlerEditorForm.CheckTableName;
begin
  if FErrorHandler.TableName = '' then
    raise Exception.Create('TableName must be defined');
end;

procedure TOraErrorHandlerEditorForm.btGenarateScriptClick(Sender: TObject);
var
  St, St1: string;
  i: integer;
begin
  CheckTableName;

  FQuery.DisableControls;
  try
    FQuery.Active := True;
    FErrorHandler.Active := True;

    Memo.Lines.Clear;

    Memo.Lines.Add('CREATE TABLE ' + FErrorHandler.TableName + ' (');
    Memo.Lines.Add('  ErrorCode INTEGER,');
    Memo.Lines.Add('  Constraint VARCHAR2(30),');
    Memo.Lines.Add('  Message VARCHAR2(100),');
    Memo.Lines.Add('  SenderName VARCHAR2(30),');
    Memo.Lines.Add('  ErrorClass VARCHAR2(20) );');
    Memo.Lines.Add('');
    FQuery.First;
    while not FQuery.EOF do begin
      Memo.Lines.Add('INSERT INTO ' + FErrorHandler.TableName +
        '(ErrorCode, Constraint, Message) VALUES');

      St := FQuery.FieldByName('Message').AsString;
      St1 := '';
      for i := 1 to Length(St) do
        if St[i] = #39 then
          St1 := St1 + ''''''
        else
          St1 := St1 + St[i];

      Memo.Lines.Add('  (' +
        IntToStr(FQuery.FieldByName('ErrorCode').AsInteger) + ', ''' +
        FQuery.FieldByName('Constraint').AsString + ''', ''' + St1 + ''');');

      FQuery.Next;
    end;
    FQuery.First;
  finally
    FQuery.EnableControls;
  end;
  PageControl.ActivePage := shMemo;
end;

procedure TOraErrorHandlerEditorForm.btCreateTableClick(Sender: TObject);
var
  OraSQL: TOraSQL;
begin
  CheckTableName;

  OraSQL := TOraSQL.Create(Self);
  try
    OraSQL.Session := FErrorHandler.Session;
    OraSQL.Debug := True;
    OraSQL.SQL.Clear;
    OraSQL.SQL.Add('CREATE TABLE ' + FErrorHandler.TableName + ' (');
    OraSQL.SQL.Add('  ErrorCode INTEGER,');
    OraSQL.SQL.Add('  Constraint VARCHAR2(30),');
    OraSQL.SQL.Add('  Message VARCHAR2(100),');
    OraSQL.SQL.Add('  SenderName VARCHAR2(30),');
    OraSQL.SQL.Add('  ErrorClass VARCHAR2(20))');

    OraSQL.Execute;

    FQuery.Active := True;
    FErrorHandler.Active := True;
  finally
    OraSQL.Free;
  end;
end;

procedure TOraErrorHandlerEditorForm.btActiveClick(Sender: TObject);
begin
  FErrorHandler.Active := not FErrorHandler.Active;
  if not FErrorHandler.Active then
    btActive.Caption := 'Open'
  else
    btActive.Caption := 'Close';
  FQuery.Active := FErrorHandler.Active;    
end;

procedure TOraErrorHandlerEditorForm.btCloseMemoClick(Sender: TObject);
begin
  PageControl.ActivePage := shGrid;
end;

{
CREATE TABLE Error (
  ErrorCode INTEGER,
  Constraint VARCHAR2(30),
  Message VARCHAR2(100),
  SenderName VARCHAR2(30),
  ErrorClass VARCHAR2(20)
)
}

procedure TOraErrorHandlerEditorForm.FormCreate(Sender: TObject);
begin
 BtnPanel.Visible := False;
end;

end.
