
//////////////////////////////////////////////////
//  Oracle Data Access Components
//  Copyright  1998-2024 Devart. All right reserved.
//  OraQuery Editor
//////////////////////////////////////////////////

{$I Odac.inc}

unit OraQueryEditor;

interface

uses
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Buttons, DBCtrls,
{$IFDEF FPC}
  LResources,
{$ENDIF}
  DASQLComponentEditor, DASQLFrame, DAParamsFrame, DAMacrosFrame, DASPCallFrame, OraSPCallFrame,
  Ora, OraError, Db, DBAccess, DASQLGenerator, {$IFNDEF FPC}MemDS{$ELSE}MemDataSet{$ENDIF},
  OraParamsFrame, CRFrame, DAQueryEditor, DASQLGeneratorFrame;

type
  TOraQueryEditorForm = class(TDAQueryEditorForm)
    shOptions: TTabSheet;
  protected
    function OptionsFrameClass: TCRFrameClass; virtual;
    procedure DoInit; override;
    procedure DoError(E: Exception); override;

    function GetFrameByInitProp: TCRFrame; override;
  public
    property Query;
  end;

implementation

{$IFNDEF FPC}
{$R *.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

uses
  CREditor, CRTabEditor, DAUpdateSQLFrame, DADataTypeMapFrame,
  OraSQLEditor, OraSQLGeneratorFrame, OraQueryOptionsFrame, DAConditionsFrame;

{ TOraQueryEditorForm }

function TOraQueryEditorForm.OptionsFrameClass: TCRFrameClass;
begin
  Result := TOraQueryOptionsFrame;
end;

procedure TOraQueryEditorForm.DoInit;
begin
  FSQLFrame := AddTab(TDASQLFrame, shSQL) as TDASQLFrame;
  FParamsFrame := AddTab(TOraParamsFrame, shParameters) as TDAParamsFrame;
  FMacrosFrame := AddTab(TDAMacrosFrame, shMacros) as TDAMacrosFrame;
  FSPCallFrame := AddTab(TOraSPCallFrame, shGeneratorSPC) as TDASPCallFrame;
  FUpdateSQLFrame := AddTab(TDAUpdateSQLFrame, shEditSQL) as TDAUpdateSQLFrame;
  FSQLGeneratorFrame := AddTab(TOraSQLGeneratorFrame, shGenerator) as TDASQLGeneratorFrame;
  FDataTypeMapFrame := AddTab(TDADataTypeMapFrame, shDataTypeMap) as TDADataTypeMapFrame;
  FConditionsFrame := AddTab(TDAConditionsFrame, shConditions) as  TDAConditionsFrame;
  AddTab(OptionsFrameClass, shOptions);

  inherited;
end;

procedure TOraQueryEditorForm.DoError(E: Exception);
begin
  if E is EOraError then
    OraSQLEditor.DoError(Self, EOraError(E))
  else
    inherited;
end;

function TOraQueryEditorForm.GetFrameByInitProp: TCRFrame;
begin
  if InitialProperty = 'SQLLock' then begin
    FUpdateSQLFrame.SetStatementType(stLock);
    Result := FUpdateSQLFrame;
  end
  else
    Result := inherited GetFrameByInitProp;
end;

end.
