
//////////////////////////////////////////////////
//  DB Access Components
//  Copyright  1998-2024 Devart. All right reserved.
//  OraQueryOptions Editor Frame
//////////////////////////////////////////////////

{$IFNDEF CLR}

{$I Odac.inc}

unit OraQueryOptionsFrame;
{$ENDIF}

interface

uses
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls,
{$IFDEF FPC}
  LResources,
{$ENDIF}
  Classes, SysUtils, CRFrame, Ora, OraSmart, OraServices;

type
  TOraQueryOptionsFrame = class(TCRFrame)
    Panel1: TPanel;
    cbLock1: TComboBox;
    Label5: TLabel;
    cbCheck: TComboBox;
    Label4: TLabel;
    Label6: TLabel;
    cbRefresh1: TComboBox;
    edKeyFields: TEdit;
    lbKeyFieldsLabel1: TLabel;
    lbUpdatingTable: TLabel;
    edUpdatingTable: TEdit;
    Memo1: TMemo;
    procedure cbCheckChange(Sender: TObject);
    procedure cbLock1Change(Sender: TObject);
    procedure cbRefresh1Change(Sender: TObject);
    procedure edKeyFieldsExit(Sender: TObject);
    procedure edUpdatingTableExit(Sender: TObject);
  protected
    function GetLocalOraQuery: TCustomOraQuery;
    procedure DoActivate; override;
    property LocalOraQuery: TCustomOraQuery read GetLocalOraQuery;
  public
    function ActiveControl: TWinControl; override;
  end;

implementation

{$IFNDEF FPC}
{$R OraQueryOptionsFrame.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

uses
  DBAccess, CREditor, OraQueryEditor;

function TOraQueryOptionsFrame.GetLocalOraQuery: TCustomOraQuery;
begin
  Result := nil;
  if FEditor is TOraQueryEditorForm then begin
    Result := TOraQueryEditorForm(FEditor).LocalComponent as TCustomOraQuery;
    Assert(Result <> nil);
  end
  else
    Assert(False);
end;

function TOraQueryOptionsFrame.ActiveControl: TWinControl;
begin
  Result := cbLock1;
end;

procedure TOraQueryOptionsFrame.DoActivate;
begin
  cbLock1.ItemIndex := Integer(LocalOraQuery.LockMode);
  cbCheck.ItemIndex := Integer(LocalOraQuery.CheckMode);
  cbRefresh1.ItemIndex := Integer(LocalOraQuery.RefreshMode);
  edUpdatingTable.Text := TDBAccessUtils.GetUpdatingTable(LocalOraQuery);
  edKeyFields.Text := LocalOraQuery.KeyFields;
end;

procedure TOraQueryOptionsFrame.cbCheckChange(Sender: TObject);
begin
  LocalOraQuery.CheckMode := TCheckMode(cbCheck.ItemIndex);
end;

procedure TOraQueryOptionsFrame.cbLock1Change(Sender: TObject);
begin
  LocalOraQuery.LockMode := Ora.TLockMode(cbLock1.ItemIndex);
end;

procedure TOraQueryOptionsFrame.cbRefresh1Change(Sender: TObject);
begin
  LocalOraQuery.RefreshMode := TRefreshMode(cbRefresh1.ItemIndex);
end;

procedure TOraQueryOptionsFrame.edKeyFieldsExit(Sender: TObject);
begin
  try
    LocalOraQuery.KeyFields := edKeyFields.Text;
  finally
    edKeyFields.Text := LocalOraQuery.KeyFields;
  end;
end;

procedure TOraQueryOptionsFrame.edUpdatingTableExit(Sender: TObject);
begin
  TDBAccessUtils.SetUpdatingTable(LocalOraQuery, edUpdatingTable.Text);
end;

end.
