
//////////////////////////////////////////////////
//  Oracle Data Access Components
//  Copyright  1998-2024 Devart. All right reserved.
//  Table SQL Frame 
//////////////////////////////////////////////////

{$IFNDEF CLR}

{$I Odac.inc}

unit OraTableSQLFrame;
{$ENDIF}

interface

uses
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  Graphics, Controls, Forms, Dialogs, StdCtrls, ExtCtrls, Buttons, ComCtrls,
{$IFDEF FPC}
  LResources,
{$ENDIF}
  Classes, SysUtils, CRTypes, DBAccess,
  CRFrame, CRTabEditor, CREditor, DATableSQLFrame;

type
  TOraTableSQLFrame = class(TDATableSQLFrame)
    cbAllTables: TCheckBox;
    lbKeyFields: TLabel;
    cbKeyFields: TComboBox;
    procedure cbAllTablesClick(Sender: TObject);
    procedure cbKeyFieldsExit(Sender: TObject);
    procedure cbKeyFieldsDropDown(Sender: TObject);
  protected
    procedure DoActivate; override;
  public
    constructor Create(Owner: TComponent); override;
  end;

implementation

{$IFNDEF FPC}
{$R OraTableSQLFrame.dfm}
{$ELSE}
{$R *.lfm}
{$ENDIF}

uses
  CRFunctions, Ora, OraSmart;

{ TOraTableSQLFrame }

constructor TOraTableSQLFrame.Create(Owner: TComponent);
begin
  inherited;

{$IFDEF UNIX}
  cbKeyFields.Items.Text := ' '; // bug in TComboBox
{$ENDIF}
end;

procedure TOraTableSQLFrame.DoActivate;
begin
  inherited;

  cbKeyFields.Text := TOraTable(GetLocalTable).KeyFields;
end;

procedure TOraTableSQLFrame.cbAllTablesClick(Sender: TObject);
begin
  FListGot := False;
  FAllTables := cbAllTables.Checked;
end;

procedure TOraTableSQLFrame.cbKeyFieldsExit(Sender: TObject);
begin
  if cbKeyFields.Text <> TOraTable(GetLocalTable).KeyFields then begin
    TOraTable(GetLocalTable).KeyFields := cbKeyFields.Text;
    InitSQL;
    Modified := True;
  end;
end;

procedure TOraTableSQLFrame.cbKeyFieldsDropDown(Sender: TObject);
var
  List: TStringList;
begin
{$IFDEF UNIX}
  cbKeyFields.OnGetItems := nil;
  try
{$ENDIF}
  if TDBAccessUtils.UsedConnection(GetLocalTable) = nil then
    Exit;

  try
    cbKeyFields.Clear;
    List := TStringList.Create;
    try
      TOraTable(GetLocalTable).GetKeyList(TOraTable(GetLocalTable).TableName, List);
      AssignStrings(List, cbKeyFields.Items);
    finally
      List.Free;
    end;
    cbKeyFields.Items.Add('ROWID');
  except
    cbKeyFields.Clear;
  {$IFDEF UNIX}
    cbKeyFields.Items.Add('');
  {$ENDIF}
    Application.HandleException(Self);
  end;
{$IFDEF UNIX}
  finally
    cbKeyFields.OnGetItems := cbKeyFieldsDropDown;
  end;
{$ENDIF}
end;

end.
