type

{ TConnectDialog }

{$IFDEF VER16P}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64{$IFNDEF STD} or pidOSX32{$IFDEF VER25P} or pidLinux64{$IFDEF VER26P} or pidOSX64{$ENDIF}{$ENDIF}{$ENDIF})]
{$ENDIF}
  {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF} = class(TCustomConnectDialog)
  private
    FRoleOption: TConnectDialogOption;
    FDirectOption: TConnectDialogOption;
    FHomeOption: TConnectDialogOption;
    FSchemaOption: TConnectDialogOption;

    procedure SetRoleOption(Value: TConnectDialogOption);
    procedure SetDirectOption(Value: TConnectDialogOption);
    procedure SetHomeOption(Value: TConnectDialogOption);
    procedure SetSchemaOption(Value: TConnectDialogOption);

    function GetSession: TOraSession;
    function GetReadAliases: boolean;
    procedure SetReadAliases(Value: boolean);

  protected
    procedure SetLabelSet(Value: TLabelSet); override;
    function GetServerEnumeratorClass: TCRServerEnumeratorClass; override;
    procedure SetServerEnumerator(Value: TCRServerEnumerator); override;

  {$IFDEF MSWINDOWS}
    function GetKeyPath: string; override;
    function GetServerStoreName: string; override;
    procedure SaveInfoToRegistry(Registry: TRegistry); override;
    procedure LoadInfoFromRegistry(Registry: TRegistry); override;
  {$ENDIF}
    function DefDialogClass: TClass; override;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;

    function Execute: boolean; override;

    property Session: TOraSession read GetSession;

    procedure GetOptions(var Options: TConnectDialogOptionArray; Ordered: Boolean = True); override;

  published
    property ReadAliases: boolean read GetReadAliases write SetReadAliases default{$IFDEF MSWINDOWS} False {$ELSE} True {$ENDIF};

    property Retries;
    property SavePassword;
    property StoreLogInfo;

    property DialogClass;

    property Caption;
    property ConnectButton;
    property CancelButton;

    property Server;
    property UserName;
    property Password;
    property Home: TConnectDialogOption read FHomeOption write SetHomeOption;
    property Direct: TConnectDialogOption read FDirectOption write SetDirectOption;
    property Schema: TConnectDialogOption read FSchemaOption write SetSchemaOption;
    property Role: TConnectDialogOption read FRoleOption write SetRoleOption;

    property LabelSet;
  end;

  function DefConnectDialogClass: TClass;

implementation

uses
{$IFDEF MSWINDOWS}
  Messages,
{$ENDIF}
{$IFDEF VER6P}
  Variants,
{$ENDIF}
{$IFDEF VER16P}
  {$IFDEF FMX}FMX.Types,{$ELSE}Controls,{$ENDIF}
{$ENDIF}
  OraServices,
  DAConsts,
  DB,
  TypInfo;

function DefConnectDialogClass: TClass;
begin
  Result := {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF};
end;

const
  KeyPath = '\SOFTWARE\Devart\ODAC\';

{ TConnectDialog }

constructor {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.Create(Owner: TComponent);
begin
  inherited Create(Owner);
  Server.Order := 1;
  UserName.Order := 2;
  Password.Order := 3;
  FHomeOption := TConnectDialogOption.Create(Self, okHome, 0, False);
  FDirectOption := TConnectDialogOption.Create(Self, okDirect, 6, False);
  FSchemaOption := TConnectDialogOption.Create(Self, okSchema, 4, False);
  FRoleOption := TConnectDialogOption.Create(Self, okRole, 5, False);
  LabelSet := lsEnglish;
end;

destructor {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.Destroy;
begin
  FRoleOption.Free;
  FDirectOption.Free;
  FHomeOption.Free;
  FSchemaOption.Free;

  inherited;
end;

function {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.Execute: boolean;
var
  OldRole: TConnectMode;
  OldDirect: Boolean;
  OldHome, OldSchema: string;
begin
  if Session = nil then
    DatabaseError(SConnectionNotDefined);
  Assert(Session is TOraSession);

  Result := False;
  OldDirect := TOraSession(Connection).Options.Direct;
  OldHome := TOraSession(Connection).HomeName;
  OldRole := TOraSession(Connection).ConnectMode;
  OldSchema := TOraSession(Connection).Schema;
  try
    Result := inherited Execute;
  finally
    if not Result then begin
      TOraSession(Connection).Options.Direct := OldDirect;
      TOraSession(Connection).HomeName := OldHome;
      TOraSession(Connection).ConnectMode := OldRole;
      TOraSession(Connection).Schema := OldSchema;
    end;
  end;
end;

procedure {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.SetLabelSet(Value: TLabelSet);
begin
  inherited SetLabelSet(Value);

  InSetLabelSet := True;
  try
    FHomeOption.Caption := 'Home Name';
    FDirectOption.Caption := 'Direct';
    FRoleOption.Caption := 'Connect Mode';
    FSchemaOption.Caption := 'Schema';
  finally
    InSetLabelSet := False;
  end;
end;

procedure {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.SetRoleOption(Value: TConnectDialogOption);
begin
  FRoleOption.Assign(Value);
end;

procedure {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.SetDirectOption(Value: TConnectDialogOption);
begin
  FDirectOption.Assign(Value);
end;

procedure {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.SetHomeOption(Value: TConnectDialogOption);
begin
  FHomeOption.Assign(Value);
end;

procedure {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.SetSchemaOption(Value: TConnectDialogOption);
begin
  FSchemaOption.Assign(Value);
end;

procedure {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.GetOptions(var Options: TConnectDialogOptionArray; Ordered: Boolean = True);
begin
  SetLength(Options, 7);
  Options[0] := UserName;
  Options[1] := Password;
  Options[2] := Server;
  Options[3] := Home;
  Options[4] := Direct;
  Options[5] := Schema;
  Options[6] := Role;
  inherited GetOptions(Options, Ordered);
end;

function {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.GetServerEnumeratorClass: TCRServerEnumeratorClass;
begin
  Result := TOraServerEnumerator;
end;

procedure {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.SetServerEnumerator(Value: TCRServerEnumerator);
begin
  inherited;

  if FServerEnumerator <> nil then begin
    if Session <> nil then begin
      FServerEnumerator.SetProp(prDirect, Session.Options.Direct);
      FServerEnumerator.SetProp(prHomeName, Session.HomeName);
//      FServerEnumerator.SetProp(prSchema, Session.Schema);
//      FServerEnumerator.SetProp(prConnectMode, Session.ConnectMode);
    end
    else begin
      FServerEnumerator.SetProp(prDirect, False);
      FServerEnumerator.SetProp(prHomeName, '');
//      FServerEnumerator.SetProp(prSchema, '');
//     FServerEnumerator.SetProp(prConnectMode, cmNormal);
    end;
  end;
end;

function {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.DefDialogClass: TClass;
begin
  Result := TConnectForm;
end;

{$IFDEF MSWINDOWS}
function {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.GetKeyPath: string;
begin
  Result := KeyPath;
end;

function {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.GetServerStoreName: string;
begin
{$IFDEF NET}
  if (Session <> nil) and Session.Options.Direct then
    Result := 'NServer'
  else
{$ENDIF}
    Result := 'Server';
end;

procedure {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.SaveInfoToRegistry(Registry: TRegistry);
begin
  inherited;

  // Home
  if Home.Visible then
    Registry.WriteString('Home', TOraSession(Connection).HomeName)
  else
    Registry.DeleteValue('Home');

  // Direct
  if Direct.Visible then
    Registry.WriteBool('Direct', TOraSession(Connection).Options.Direct)
  else
    Registry.DeleteValue('Direct');

  // Schema
  if Schema.Visible then
    Registry.WriteString('Schema', TOraUtils.FSchema(TOraSession(Connection)))
  else
    Registry.DeleteValue('Schema');

  // Role
  if Role.Visible then
    Registry.WriteString('Role', GetEnumName(TypeInfo(TConnectMode), integer(TOraSession(Connection).ConnectMode)))
  else
    Registry.DeleteValue('Role');
end;

procedure {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.LoadInfoFromRegistry(Registry: TRegistry);
begin
  // Direct
  if Direct.Visible and Registry.ValueExists('Direct') then
    TOraSession(Connection).Options.Direct := Registry.ReadBool('Direct');

  inherited;

  // Home
  if Home.Visible and Registry.ValueExists('Home') then
    TOraSession(Connection).HomeName := Registry.ReadString('Home');

  // Schema
  if Schema.Visible and Registry.ValueExists('Schema') then
    TOraSession(Connection).Schema := Registry.ReadString('Schema');

  // Role
  if Role.Visible and Registry.ValueExists('Role') then
    TOraSession(Connection).ConnectMode := TConnectMode(GetEnumValue(TypeInfo(TConnectMode), Registry.ReadString('Role')));
end;
{$ENDIF}

function {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.GetSession: TOraSession;
begin
  Result := TOraSession(Connection);
end;

function {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.GetReadAliases: boolean;
begin
  Result := not UseServerHistory;
end;

procedure {$IFDEF FMX}TConnectDialogFmx{$ELSE}TConnectDialog{$ENDIF}.SetReadAliases(Value: boolean);
begin
  UseServerHistory := not Value;
end;

initialization
{$IFDEF FMX}
  if not Assigned(DefConnectDialogClassProc) then
{$ENDIF}
    DefConnectDialogClassProc := DefConnectDialogClass;

{$IFDEF VER16P}
{$IFDEF FMX}
  GroupDescendentsWith(TConnectDialogFmx, TFmxObject);
{$ELSE}
  GroupDescendentsWith(TConnectDialog, TControl);
{$ENDIF}
{$ENDIF}

end.
